﻿#define F_CPU 16000000UL // Define CPU frequency for delay function


#include <avr/io.h>
#include <util/delay.h>


// LCD Pin Definitions
#define LCD_DATA_PORT PORTA // LCD data port
#define LCD_DATA_DDR DDRA   // LCD data direction register
#define LCD_RS_PORT PORTB   // LCD RS port
#define LCD_RS_DDR DDRB     // LCD RS direction register
#define LCD_RS_PIN 0        // LCD RS pin
#define LCD_EN_PORT PORTB   // LCD EN port
#define LCD_EN_DDR DDRB     // LCD EN direction register
#define LCD_EN_PIN 1        // LCD EN pin


// ADC Pin Definitions
#define ADC_CH 0          // ADC channel for ACS712 sensor
#define ADC_REFERENCE 5   // ADC reference voltage in volts (5V)


// PWM Pin Definitions
#define PWM_PIN 3         // PWM output pin for L298N motor driver


// Potentiometer Pin Definition
#define POT_PIN 2         // Analog input pin for potentiometer


// Function prototypes
void lcd_cmd(unsigned char cmd);
void lcd_data(unsigned char data);
void lcd_init();
void lcd_clear();
void lcd_set_cursor(unsigned char row, unsigned char col);
void lcd_print_string(char *str);
void lcd_print_int(int num);
void adc_init();
uint16_t adc_read();
void pwm_init();
void pwm_set_duty_cycle(uint8_t duty_cycle);


int main(void)
{
        // Initialize LCD, ADC, and PWM
        lcd_init();
        adc_init();
        pwm_init();


        // Variables for storing ADC readings and potentiometer value
        uint16_t adc_value;
        uint8_t pot_value;


        // Main loop
        while (1)
        {
                // Read current from ACS712 sensor
                adc_value = adc_read();


                // Update LCD with current reading
                lcd_clear();
                lcd_set_cursor(0, 0);
                lcd_print_string("Current (A):");
                lcd_set_cursor(1, 0);
                lcd_print_int(adc_value);
                _delay_ms(500); // Update LCD every 0.5 seconds


                // Read potentiometer value for controlling water pump
                pot_value = ADC >> 2; // Scale potentiometer value to 0-255 (8-bit)


                // Set PWM duty cycle based on potentiometer value
                pwm_set_duty_cycle(pot_value);
        }
}


void lcd_cmd(unsigned char cmd)
{
        LCD_DATA_PORT = cmd;
        LCD_RS_PORT &= ~(1 << LCD_RS_PIN); // RS = 0 for command
        LCD_EN_PORT |= (1 << LCD_EN_PIN);  // EN = 1 to enable LCD
        _delay_us(1);                      // Delay for EN pulse width
        LCD_EN_PORT &= ~(1 << LCD_EN_PIN); // EN = 0 to disable LCD
        _delay_ms(2);                      // Delay for LCD command execution
}


void lcd_data(unsigned char data)
{
        LCD_DATA_PORT = data;
        LCD_RS_PORT |= (1 << LCD_RS_PIN); // RS = 1 for data
        LCD_EN_PORT |= (1 << LCD_EN_PIN); // EN = 1 to enable LCD
        _delay_us(1);                     // Delay for EN pulse width
        LCD_EN_PORT &= ~(1 << LCD_EN_PIN);// EN = 0 to disable LCD
        _delay_us(100);                   // Delay for LCD data write time
}


void lcd_init()
{
        LCD_DATA_DDR = 0xFF;   // Set LCD data port as output
        
    LCD_RS_DDR |= (1 << LCD_RS_PIN); // Set LCD RS pin as output
    LCD_EN_DDR |= (1 << LCD_EN_PIN); // Set LCD EN pin as output


    _delay_ms(20); // Delay for LCD power-up


    // Initialize LCD in 4-bit mode
    lcd_cmd(0x02); // Set 4-bit mode
    lcd_cmd(0x28); // 2 lines, 5x8 character size
    lcd_cmd(0x06); // Increment cursor, no display shift
    lcd_cmd(0x0C); // Display on, cursor off, blinking off
    lcd_cmd(0x01); // Clear display
    _delay_ms(2);  // Delay for LCD command execution
    }


    void lcd_clear()
    {
            lcd_cmd(0x01); // Clear display
            _delay_ms(2);  // Delay for LCD command execution
    }


    void lcd_set_cursor(unsigned char row, unsigned char col)
    {
            unsigned char addr;


            // Calculate DDRAM address based on row and col
            if (row == 0)
            {
                    addr = 0x80 + col; // First row starts at address 0x80
            }
            else
            {
                    addr = 0xC0 + col; // Second row starts at address 0xC0
            }


            lcd_cmd(addr); // Set DDRAM address
    }


    void lcd_print_string(char *str)
    {
            while (*str)
            {
                    lcd_data(*str++);
            }
    }


    void lcd_print_int(int num)
    {
            char str[7]; // Buffer for storing converted string
            sprintf(str, "%d", num); // Convert integer to string
            lcd_print_string(str); // Print string to LCD
    }


    void adc_init()
    {
            ADMUX |= (1 << REFS0); // Set ADC reference voltage to AVCC
            ADCSRA |= (1 << ADEN) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // Enable ADC, set prescaler to 128
    }


    uint16_t adc_read()
    {
            ADCSRA |= (1 << ADSC); // Start ADC conversion
            while (ADCSRA & (1 << ADSC)); // Wait for ADC conversion to complete
            return ADC; // Return ADC value
    }


    void pwm_init()
    {
            DDRD |= (1 << PWM_PIN); // Set PWM pin as output
            TCCR0A |= (1 << WGM00) | (1 << WGM01) | (1 << COM0A1); // Fast PWM, non-inverted mode
            TCCR0B |= (1 << CS01); // Set prescaler to 8
    }


    void pwm_set_duty_cycle(uint8_t duty_cycle)
    {
            OCR0A = duty_cycle; // Set duty cycle value to OCR0A register
    }